"""REST client handling, including QualifiedStream base class."""
from __future__ import annotations

from pathlib import Path
from typing import Any, Dict, Iterable, Optional

import requests
from singer_sdk.authenticators import BearerTokenAuthenticator
from singer_sdk.helpers.jsonpath import extract_jsonpath
from singer_sdk.pagination import BaseAPIPaginator, JSONPathPaginator
from singer_sdk.streams import RESTStream

SCHEMAS_DIR = Path(__file__).parent / Path("./schemas")


class ShowpadPaginator(JSONPathPaginator):
    """Showpad API paginator class."""

    next_page_path = "$.meta.ending_cursor"
    has_more_path = "$.meta.has_more"

    def get_next(self, response: requests.Response) -> str | None:
        return next(extract_jsonpath(self.next_page_path, response.json()), None)

    def has_more(self, response: requests.Response) -> bool:
        return bool(next(extract_jsonpath(self.has_more_path, response.json()), False))


class QualifiedStream(RESTStream):
    """Qualified stream class."""

    url_base = "https://api.qualified.com/v1"
    records_jsonpath = "$.data[*]"

    @property
    def authenticator(self) -> BearerTokenAuthenticator:
        """Return a new authenticator object."""
        return BearerTokenAuthenticator.create_for_stream(
            self, token=self.config.get("api_key")
        )

    @property
    def http_headers(self) -> dict:
        """Return the http headers needed."""
        headers = {}
        if "user_agent" in self.config:
            headers["User-Agent"] = self.config.get(
                "user_agent", "Harness.io/Tap Qualified"
            )
        return headers

    def get_new_paginator(self) -> BaseAPIPaginator:
        """Return a token for identifying next page or None if no more pages."""
        return ShowpadPaginator(...)

    def get_url_params(
        self, context: Optional[dict], next_page_token: Optional[Any]
    ) -> Dict[str, Any]:
        """Return a dictionary of values to be used in URL parameterization."""
        params: dict = {}
        if next_page_token:
            params["starting_after"] = next_page_token
        return params

    def parse_response(self, response: requests.Response) -> Iterable[dict]:
        """Parse the response and return an iterator of result records."""
        yield from extract_jsonpath(self.records_jsonpath, input=response.json())



"""Tests standard tap features using the built-in SDK tests library."""
import os

from singer_sdk.testing import get_tap_test_class

from tap_qualified.tap import TapQualified

BASE_CONFIG = {
    "api_key": os.getenv("TAP_QUALIFIED_API_KEY"),
}

# Run standard built-in tap tests from the SDK:
TestTapQualified = get_tap_test_class(tap_class=TapQualified, config=BASE_CONFIG)



"""Qualified tap class."""

from typing import List

from singer_sdk import Stream, Tap
from singer_sdk import typing as th

from tap_qualified.streams import BotConversationStream, RepConversationStream


class TapQualified(Tap):
    """Qualified tap class."""

    name = "tap-qualified"

    config_jsonschema = th.PropertiesList(
        th.Property(
            "api_key",
            th.StringType,
            required=True,
            secret=True,
            description="The token to authenticate against the API service",
        ),
        th.Property(
            "user_agent",
            th.StringType,
            description="The user agent to use when making requests to the API service",
            default="Harness.io/Tap Qualified",
        ),
    ).to_dict()

    def discover_streams(self) -> List[Stream]:
        """Return a list of discovered streams."""
        return [
            stream_class(tap=self)
            for stream_class in (BotConversationStream, RepConversationStream)
        ]


if __name__ == "__main__":
    TapQualified.cli()



"""Stream type classes for tap-qualified."""
from pathlib import Path

from tap_qualified.client import QualifiedStream

SCHEMAS_DIR = Path(__file__).parent / Path("./schemas")


class RepConversationStream(QualifiedStream):
    """A stream of conversations with reps."""

    name = "rep_conversations"
    path = "/rep_conversations"
    primary_keys = ["id"]
    schema_file = SCHEMAS_DIR / "rep_conversations.json"


class BotConversationStream(QualifiedStream):
    """A stream of conversations with bots."""

    name = "bot_conversations"
    path = "/bot_conversations"
    primary_keys = ["id"]
    schema_file = SCHEMAS_DIR / "bot_conversations.json"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tap_qualified', 'tap_qualified.tests']

package_data = \
{'': ['*'], 'tap_qualified': ['schemas/*']}

install_requires = \
['requests>=2.28.1', 'singer-sdk>=0.19.0,<1']

extras_require = \
{'s3': ['fs-s3fs>=1.1.1']}

entry_points = \
{'console_scripts': ['tap-qualified = tap_qualified.tap:TapQualified.cli']}

setup_kwargs = {
    'name': 'tap-qualified',
    'version': '1.0.0',
    'description': '`tap-qualified` is a Singer tap for Qualified, built with the Meltano Singer SDK.',
    'long_description': "# tap-qualified\n\n`tap-qualified` is a Singer tap for [Qualified](https://www.qualified.com/).\n\nQualified helps companies generate pipeline, faster. Tap into your greatest asset - your website - to identify your most valuable visitors, instantly start sales conversations, schedule  meetings, convert outbound and paid traffic, and uncover signals of buying intent.\n\nInstall from PyPi:\n\n```bash\npipx install tap-qualified\n```\n\n## Configuration\n\n### Accepted Config Options\n\n| Setting             | Required | Default | Description |\n|:--------------------|:--------:|:-------:|:------------|\n| api_key             | True     | None    | The token to authenticate against the API service |\n| user_agent          | False    | Harness.io/Tap Qualified | The user agent to use when making requests to the API service |\n| stream_maps         | False    | None    | Config object for stream maps capability. For more information check out [Stream Maps](https://sdk.meltano.com/en/latest/stream_maps.html). |\n| stream_map_config   | False    | None    | User-defined config values to be used within map expressions. |\n| flattening_enabled  | False    | None    | 'True' to enable schema flattening and automatically expand nested properties. |\n| flattening_max_depth| False    | None    | The max depth to flatten schemas. |\n\nA full list of supported settings and capabilities for this\ntap is available by running:\n\n```bash\ntap-qualified --about\n```\n\n### Configure using environment variables\n\nThis Singer tap will automatically import any environment variables within the working directory's\n`.env` if the `--config=ENV` is provided, such that config values will be considered if a matching\nenvironment variable is set either in the terminal context or in the `.env` file.\n\n### Source Authentication and Authorization\n\nReach out to your account executive to inquire about API access.\n\n## Usage\n\nYou can easily run `tap-qualified` by itself or in a pipeline using [Meltano](https://meltano.com/).\n\n### Executing the Tap Directly\n\n```bash\ntap-qualified --version\ntap-qualified --help\ntap-qualified --config CONFIG --discover > ./catalog.json\n```\n\n## Developer Resources\n\nFollow these instructions to contribute to this project.\n\n### Initialize your Development Environment\n\n```bash\npipx install poetry\npoetry install\n```\n\n### Create and Run Tests\n\nCreate tests within the `tap_qualified/tests` subfolder and\n  then run:\n\n```bash\npoetry run pytest\n```\n\nYou can also test the `tap-qualified` CLI interface directly using `poetry run`:\n\n```bash\npoetry run tap-qualified --help\n```\n\n### Testing with [Meltano](https://www.meltano.com)\n\n_**Note:** This tap will work in any Singer environment and does not require Meltano.\nExamples here are for convenience and to streamline end-to-end orchestration scenarios._\n\n\nNext, install Meltano (if you haven't already) and any needed plugins:\n\n```bash\n# Install meltano\npipx install meltano\n# Initialize meltano within this directory\ncd tap-qualified\nmeltano install\n```\n\nNow you can test and orchestrate using Meltano:\n\n```bash\n# Test invocation:\nmeltano invoke tap-qualified --version\n# OR run a test `elt` pipeline:\nmeltano elt tap-qualified target-jsonl\n```\n\n### SDK Dev Guide\n\nSee the [dev guide](https://sdk.meltano.com/en/latest/dev_guide.html) for more instructions on how to use the SDK to\ndevelop your own taps and targets.\n\nBuilt with the [Meltano Tap SDK](https://sdk.meltano.com) for Singer Taps.\n",
    'author': 'Alex Butler',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/z3z1ma/tap-qualified',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.12',
}


setup(**setup_kwargs)
